//
//  LineGraph.m
//
//  Created by Robert on 04/10/2009.
//

#import "LineGraph.h"

// --------------------------------------------------------
// The Line Object
// --------------------------------------------------------
@implementation Line 

#pragma mark ---------------------------------------------------------
#pragma mark === Properties  ===
#pragma mark ---------------------------------------------------------

@synthesize name;
@synthesize values;
@synthesize color;
@synthesize width;
@synthesize dashed;

#pragma mark ---------------------------------------------------------
#pragma mark === End Properties  ===
#pragma mark ---------------------------------------------------------

#pragma mark ---------------------------------------------------------
#pragma mark === Constructor / Destructor Functions  ===
#pragma mark ---------------------------------------------------------

// -------------------------------------------------------------------
// initialization
// -------------------------------------------------------------------
- (id) initWithName:(NSString*)_name AndColor:(UIColor*)_color AndWidth:(float)_width
{
	return [self initWithName:_name AndColor:_color AndWidth:_width SetDashed:FALSE];
}

// -------------------------------------------------------------------
// initialization
// -------------------------------------------------------------------
- (id) initWithName:(NSString*)_name AndColor:(UIColor*)_color AndWidth:(float)_width SetDashed:(Boolean)_dashed
{
	if ( (self = [super init]) )
	{
		name		= [_name copy];
		values		= [[NSMutableArray alloc] initWithCapacity:32];
		self.color	= _color;
		width		= _width;
		dashed		= _dashed;
	}
	
	return self;
}

// -------------------------------------------------------------------
//  destructor
// -------------------------------------------------------------------
- (void)dealloc 
{
	[values release];
	[color release];
    [super dealloc];
}

#pragma mark ---------------------------------------------------------
#pragma mark === End Constructor / Destructor Functions  ===
#pragma mark ---------------------------------------------------------

// -------------------------------------------------------------------
// add a value
// -------------------------------------------------------------------
- (void) add:(float)_value
{
	[values addObject:[NSNumber numberWithFloat:_value]];
}

// -------------------------------------------------------------------
// remove the first value
// -------------------------------------------------------------------
- (void) pop
{
	if ( [values count] > 0 )
	{
		[values removeObjectAtIndex:0];
	}
}

// -------------------------------------------------------------------
// clear all the values
// -------------------------------------------------------------------
- (void) clear
{
	[values removeAllObjects];
}

@end
// --------------------------------------------------------

// --------------------------------------------------------
// Private Methods
// --------------------------------------------------------
@interface LineGraph (PrivateMehtods)

- (void) initialise;
- (void) createTestData;

- (void) calculateValues;
- (float) calculateLineHeightFromRect:(CGRect)rect;

- (CGRect) orientateContext:(CGContextRef)context AndRect:(CGRect)rect;
- (CGRect) renderGraphKeyInContext:(CGContextRef)context AndRect:(CGRect)rect;
- (CGRect) renderLabelsInContext:(CGContextRef)context AndRect:(CGRect)rect;
- (CGRect) renderArrayLabelsInContext:(CGContextRef)context AndRect:(CGRect)rect;
- (CGRect) renderScaleValuesInContext:(CGContextRef)context AndRect:(CGRect)rect;
- (void)   renderGraphBackGroundInContext:(CGContextRef)context AndRect:(CGRect)rect;
- (void)   renderGraphMarkerLineInContext:(CGContextRef)context AndRect:(CGRect)rect;

@end
// --------------------------------------------------------

@implementation LineGraph

// --------------------------------------------------------
// Text rotations
// --------------------------------------------------------
const CGAffineTransform lineTextTransform = { 1.0, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f };
const CGAffineTransform lineTextRotTransform = { 0.0, -1.0f, -1.0f, 0.0f, 0.0f, 0.0f };
// --------------------------------------------------------

#pragma mark ---------------------------------------------------------
#pragma mark === Properties  ===
#pragma mark ---------------------------------------------------------

// -------------------------------------------------------------------
// Property : Return the number of lines contained
// -------------------------------------------------------------------
- (NSInteger) count
{
	return [values count];
}

- (NSInteger) maxNumEntries
{
	return maxNumEntries;
}

- (void) setMaxNumEntries:(NSInteger)_maxEntries
{
	if ( _maxEntries < minNumEntries )
	{
		maxNumEntries = minNumEntries;
	}
	else
	{
		maxNumEntries = _maxEntries;
	}
}

- (NSInteger) minNumEntries
{
	return minNumEntries;
}

- (void) setMinNumEntries:(NSInteger)_minEntries
{
	if ( _minEntries <= 0 )
	{
		minNumEntries = 1;
	}
	else 
	{
		minNumEntries = _minEntries;
		
		if ( maxNumEntries < minNumEntries )
		{
			// minNumEntries = maxNumEntries;
			maxNumEntries = minNumEntries;
		}
	}	
}

@synthesize backgroundImage;
@synthesize title;
@synthesize verticalAxisName;
@synthesize horizontalAxisName;
@synthesize orientation;

@synthesize indent;
@synthesize font;
@synthesize fontSmall;
@synthesize fontColor;
@synthesize lineWidth;
@synthesize lineColor;
@synthesize gridLineWidth;
@synthesize gridLineColor;
@synthesize fillColor;
@synthesize lineHighlightPercentage;
@synthesize lineHighlightColor;
@synthesize lineShadowOffset;
@synthesize lineShadowColor;

@synthesize drawGrid;
@synthesize drawEdge;
@synthesize drawLine;
@synthesize drawFill;
@synthesize drawScale;
@synthesize drawKey;

#pragma mark ---------------------------------------------------------
#pragma mark === End Properties  ===
#pragma mark ---------------------------------------------------------

#pragma mark ---------------------------------------------------------
#pragma mark === Constructor / Destructor Functions  ===
#pragma mark ---------------------------------------------------------

// -------------------------------------------------------------------
//  Initialisation
// -------------------------------------------------------------------
- (id)initWithFrame:(CGRect)frame 
{
    if ((self = [super initWithFrame:frame])) 
	{
        // Initialization code
		[self initialise];
    }
    return self;
}

// -------------------------------------------------------------------
// Initialisation
// -------------------------------------------------------------------
- (id)initWithCoder:(NSCoder*)coder 
{	
	if ( (self = [super initWithCoder:coder]) )
	{
		// Initialization code
		[self initialise];
	}
	return self;
}

// -------------------------------------------------------------------
// Initialise with the defaults
// -------------------------------------------------------------------
- (void) initialise
{
	values = [[NSMutableArray alloc] initWithCapacity:10];
	labels = [[NSMutableArray alloc] initWithCapacity:10];
	
	orientation				= LineGraphOrientationUnknown;
	maxNumEntries			= 10;
	minNumEntries			= 10;
	
	maxValue				= 100.0f;
	minValue				= 0.0f;
	totalValue				= 0.0f;
	
	indent.width			= 30;
	indent.height			= 30;
	
	font					= [[UIFont fontWithName:@"Helvetica" size:12] retain];
	fontSmall				= [[UIFont fontWithName:@"Helvetica" size:10] retain];
	fontColor				= [[UIColor blackColor] retain];
	
	lineWidth				= 2;
	lineColor				= [[UIColor blackColor] retain];
	
	gridLineWidth			= 1;
	gridLineColor			= [[UIColor blackColor] retain];
	
	fillColor				= [[UIColor colorWithRed:0 green:0 blue:0 alpha:0.1f] retain];
	
	lineHighlightPercentage	= 0.5f;
	lineHighlightColor		= [[UIColor colorWithWhite:1 alpha:0.35f] retain]; 
	
	lineShadowColor			= [[UIColor colorWithRed:0 green:0 blue:0 alpha:0.5f] retain];
	lineShadowOffset		= CGPointMake( 2, 2 );
	
	drawLine				= TRUE;
	drawGrid				= TRUE;
	drawEdge				= TRUE;
	drawFill				= TRUE;
	drawScale				= TRUE;
	drawKey					= TRUE;
	
	// remove the test data
	//[self createTestData];
}

// -------------------------------------------------------------------
// Create the Test Data
// -------------------------------------------------------------------
- (void) createTestData
{
	self.verticalAxisName	= @"Vertical Axis";
	self.horizontalAxisName = @"Horazontal Axis";
	self.title              = @"Title"; 
	
	Line * line = [self addLineWithName:@"Test0" WithColor:[UIColor redColor] AndWidth:4];
	[line add:10];
	[line add:5];
	[line add:-15];
	[line add:15];
	[line add:5];
	[line add:-10];
	[line add:-1];
	
	line = [self addLineWithName:@"Test1" WithColor:[UIColor blueColor] AndWidth:4 SetDashed:TRUE];
	[line add:1];
	[line add:5];
	[line add:-10];
	[line add:20];
	[line add:3];
	[line add:-13];
	[line add:14];
}

// -------------------------------------------------------------------
//  destructor
// -------------------------------------------------------------------
- (void)dealloc 
{
	[values release];
	[labels release];
	[font release];
	[fontSmall release];
	[fillColor release];
	[lineColor release];
	[gridLineColor release];
	[lineHighlightColor release];
	[lineShadowColor release];
	
    [super dealloc];
}

#pragma mark ---------------------------------------------------------
#pragma mark === End Constructor / Destructor Functions  ===
#pragma mark ---------------------------------------------------------

#pragma mark ---------------------------------------------------------
#pragma mark === View Functions  ===
#pragma mark ---------------------------------------------------------

// -------------------------------------------------------------------
// draw rect
// -------------------------------------------------------------------
- (void)drawRect:(CGRect)rect 
{
    // Drawing code
	[super drawRect:rect];
		
	// ----------------------------------------
	// Grab the context
	// ----------------------------------------
	CGContextRef context = UIGraphicsGetCurrentContext();
	// ----------------------------------------
	
	
	// ----------------------------------------
	// Calculate the min and max values
	// ----------------------------------------
	[self calculateValues];
	// ----------------------------------------
	
	// ----------------------------------------
	// Orientate the context to the desired
	// rotation and correct the rect
	// ----------------------------------------
	rect = [self orientateContext:context AndRect:rect];
	// ----------------------------------------
	
	// ----------------------------------------
	// draw the background image
	// ----------------------------------------
	if ( backgroundImage )
	{
		CGContextDrawImage( context, rect, [backgroundImage CGImage] );
	}
	// ----------------------------------------
	
	// ----------------------------------------
	// Work out the bar draw area
	// ----------------------------------------	
	rect.origin.x += indent.width;
	rect.origin.y += indent.height;
	rect.size.width -= ( indent.width * 2 );
	rect.size.height -= ( indent.height * 2 );
	// ----------------------------------------

	// ----------------------------------------
	// draw of the key area
	// ----------------------------------------
	if ( drawKey )
	{
		rect = [self renderGraphKeyInContext:context AndRect:rect];
	}
	// ----------------------------------------
	
	// ----------------------------------------
	// Draw the axis labels
	// ----------------------------------------
	rect = [self renderLabelsInContext:context AndRect:rect];
	// ----------------------------------------
	
	// ----------------------------------------
	// Draw an array of labels along the bottom
	// of the graph
	// ----------------------------------------
	rect = [self renderArrayLabelsInContext:context AndRect:rect];
	// ----------------------------------------
	
	// ----------------------------------------
	// draw the scale values
	// ----------------------------------------
	if ( drawScale )
	{
		rect = [self renderScaleValuesInContext:context AndRect:rect];
	}
	// ----------------------------------------
	
	// ----------------------------------------
	// draw the Background
	// ----------------------------------------
	[self renderGraphBackGroundInContext:context AndRect:rect];
	// ----------------------------------------
	
	// ----------------------------------------
	// draw the marker line
	// ----------------------------------------
	[self renderGraphMarkerLineInContext:context AndRect:rect];
	// ----------------------------------------
	
	// ----------------------------------------
	// draw the lines
	// ----------------------------------------
	{
		float lineHeight = [self calculateLineHeightFromRect:rect];
		
		float areawidth		= rect.size.width;
		float maxLineWidth	= 0;
		for (Line * obj in values) 
		{
			if ( obj.width > maxLineWidth )
			{
				maxLineWidth = obj.width;
			}
		}
		areawidth -= maxLineWidth;
		
		// limit the spread of the points
		NSInteger entries = peakEntries;
		if ( minNumEntries > peakEntries )
		{
			entries = minNumEntries;
		}
		else if ( peakEntries > maxNumEntries )
		{
			entries = maxNumEntries;
		}
		
		float stepDistance = areawidth / ( entries - 1 );
		float height       = ( ( rect.size.height - lineWidth ) * ( ( minValue >= 0.0f ) ? 1.0f : 0.5f ) ); // last 2 brings it in from the edge
		
		CGContextSetLineCap(context, kCGLineCapRound);
		CGContextSetLineJoin(context, kCGLineJoinRound);
		
		// render each individual line
		for (Line * obj in values) 
		{
			if ( [obj.values count] > 0 )
			{				
				// move to the start point
				CGPoint point  = rect.origin;
				point.x +=  obj.width * 0.5f;
				
				if ( [obj.values count] == 1 )
				{
					// ----------------------------------------------
					// scale the points position
					// ----------------------------------------------
					NSNumber * num	= [obj.values objectAtIndex:0];
					float value		= [num floatValue];
					float scale		= ( value / ( ( value >= 0 ) ? maxValue : -minValue ) );
					float pointY	= lineHeight + ( (lineWidth * 0.5f) * ( ( value >= 0.0f ) ? -1 : 1 ) );

					point.y = pointY - ( height * scale );
					// ----------------------------------------------
					
					// ----------------------------------------------
					// work out the point radius 
					// this will be used to shift the rect around
					// ----------------------------------------------
					float radius = obj.width*0.5f;
					// ----------------------------------------------
					
					// ----------------------------------------------
					// draw the shadow
					// ----------------------------------------------
					CGContextSetFillColorWithColor(context, [lineShadowColor CGColor]);
					CGContextFillEllipseInRect(context, CGRectMake(point.x + lineShadowOffset.x - radius, point.y + lineShadowOffset.y-radius, obj.width, obj.width));
					// ----------------------------------------------
					
					// ----------------------------------------------
					// draw the primary point
					// ----------------------------------------------
					CGContextSetFillColorWithColor(context, [obj.color CGColor]);
					CGContextFillEllipseInRect(context, CGRectMake(point.x-radius, point.y-radius, obj.width, obj.width));
					// ----------------------------------------------
					
					// ----------------------------------------------
					// draw the highlight
					// ----------------------------------------------
					float width = (float)(obj.width) * lineHighlightPercentage;
					if ( width > 1 )
					{
						CGContextSetFillColorWithColor(context, [lineHighlightColor CGColor]);
						CGContextFillEllipseInRect(context, CGRectMake(point.x-radius, point.y-(obj.width - width)-radius, width, width));
					}			
					// ----------------------------------------------
				}
				else 
				{
					NSInteger total = [obj.values count];
				
					// cap the entries that are drawn
					if ( total > maxNumEntries )
					{
						total = maxNumEntries;
					}
					// This could also be done
					// by always showing the end
					// off the line either by
					// poping the extra entries 
					// or by starting the index
					// at a diffrent position
					
					NSNumber * num  = nil;//[obj.values objectAtIndex:0];
					float value     = 0;//[num floatValue];
					float scale		= 0;//( value / ( ( value >= 0 ) ? maxValue : -minValue ) );
					int pointY	    = 0;//lineHeight + ( (lineWidth * 0.5f) * ( ( value >= 0.0f ) ? -1 : 1 ) );
					
					CGPoint path[total];
					for ( int i=0; i<total; i++ )
					{
						num			= [obj.values objectAtIndex:i];
						value		= [num floatValue];
						scale		= ( value / ( ( value >= 0 ) ? maxValue : -minValue ) );
						pointY	    = lineHeight + ( (lineWidth * 0.5f) * ( ( value >= 0.0f ) ? -1 : 1 ) );
						
						
						point.y = pointY - ( height * scale );
						path[i] = point;
						point.x += stepDistance;
					}
					
					if ( obj.dashed )
					{
						float dashlen   = obj.width;
						float lengths[] = {dashlen, dashlen*1.5f};
						CGContextSetLineDash(context, 0.0, lengths, 2);
					}
					
					// ----------------------------------------------
					// draw the shadow
					// ----------------------------------------------
					CGContextSetStrokeColorWithColor(context, [lineShadowColor CGColor]);
					CGContextSetLineWidth(context, obj.width);
									
					CGContextSaveGState(context);
					CGContextTranslateCTM(context, lineShadowOffset.x, lineShadowOffset.y);
					CGContextAddLines(context, path, total);
					CGContextSetLineWidth(context, obj.width);
					CGContextStrokePath(context);
					CGContextRestoreGState(context);
					// ----------------------------------------------

					// ----------------------------------------------
					// draw the primary line
					// ----------------------------------------------
					CGContextSetLineWidth(context, obj.width);
					CGContextSetStrokeColorWithColor(context, [obj.color CGColor]);
					CGContextBeginPath(context);
					CGContextAddLines(context, path, total);
					CGContextStrokePath(context);
					CGContextClosePath(context);
					// ----------------------------------------------
					
					// ----------------------------------------------
					// draw the highlight
					// ----------------------------------------------
					float width = (float)(obj.width) * lineHighlightPercentage;
					if ( width > 1 )
					{
						CGContextSetStrokeColorWithColor(context, [lineHighlightColor CGColor]);
						CGContextSetLineWidth(context, width);
						
						CGContextSaveGState(context);
						CGContextTranslateCTM(context, 0, obj.width - width);
						CGContextBeginPath(context);
						CGContextAddLines(context, path, total);
						CGContextStrokePath(context);
						CGContextClosePath(context);
						CGContextRestoreGState(context);					
					}			
					// ----------------------------------------------
					
					// ----------------------------------------------
					// Remove the dashed line by setting it to NULL
					// if there is no dashed line present this will
					// do nothing.
					// ----------------------------------------------
					CGContextSetLineDash(context, 0.0, NULL, 0);
					// ----------------------------------------------
				}
			}
		}
	}
	// ----------------------------------------
	
}

#pragma mark ---------------------------------------------------------
#pragma mark === End View Functions  ===
#pragma mark ---------------------------------------------------------

#pragma mark ---------------------------------------------------------
#pragma mark === Public Functions  ===
#pragma mark ---------------------------------------------------------

// -------------------------------------------------------------------
// Function : Add a line to graph
// -------------------------------------------------------------------
- (Line*) addLineWithName:(NSString*)_name WithColor:(UIColor*)_color AndWidth:(float)_width
{
	Line * line = [[Line alloc] initWithName:_name  AndColor:_color AndWidth:_width];
	[values addObject:line];
	[line release];
	return line;
}

// -------------------------------------------------------------------
// Function : Add a line to graph
// -------------------------------------------------------------------
- (Line*) addLineWithName:(NSString*)_name WithColor:(UIColor*)_color AndWidth:(float)_width SetDashed:(Boolean)_dashed
{
	Line * line = [[Line alloc] initWithName:_name  AndColor:_color AndWidth:_width SetDashed:_dashed];
	[values addObject:line];
	[line release];
	return line;
}

// -------------------------------------------------------------------
// Function : return the line at the index passed in
// -------------------------------------------------------------------
- (Line*) lineAtIndex:(NSInteger)_index
{
	return [values objectAtIndex:_index];
}

// -------------------------------------------------------------------
// Function : return the first line with the name that was passed in
// ------------------------------------------------------------------- 
- (Line*) lineWithName:(NSString*)_name
{
	for ( Line * obj in values )
	{
		if ( [obj.name compare:_name] == NSOrderedSame )
		{
			return obj;
		}
	}
	
	return nil;
}

// -------------------------------------------------------------------
// remove all the current lines objects
// -------------------------------------------------------------------
- (void) removeAllLines
{
	[values removeAllObjects];
}

// -------------------------------------------------------------------
// Function : Remove the line at the index provided
// -------------------------------------------------------------------
- (void) removeLineAtIndex:(NSInteger)_index
{
	[values removeObjectAtIndex:_index];
}

// -------------------------------------------------------------------
// Function : Remove the first line found with the name provided
// -------------------------------------------------------------------
- (void) removeLineWithName:(NSString*)_name
{
	Line * line = nil;
	for ( Line * obj in values )
	{
		if ( [obj.name compare:_name] == NSOrderedSame )
		{
			line = obj;
			break;
		}
	}
	
	[self removeLine:line];
}

// -------------------------------------------------------------------
// Function : Remove the line object provided
// -------------------------------------------------------------------
- (void) removeLine:(Line*)_line
{
	[values removeObject:_line];
}

// -------------------------------------------------------------------
// add Label
// -------------------------------------------------------------------
- (void) addLabel:(NSString*)_text
{
	[labels addObject:_text];
}

// -------------------------------------------------------------------
// remove all the labels
// -------------------------------------------------------------------
- (void) clearLabels
{
	[labels removeAllObjects];
}

#pragma mark ---------------------------------------------------------
#pragma mark === End Public Functions  ===
#pragma mark ---------------------------------------------------------

#pragma mark ---------------------------------------------------------
#pragma mark === Private Functions  ===
#pragma mark ---------------------------------------------------------

// -------------------------------------------------------------------
// Calculate the Graph Values
// -------------------------------------------------------------------
- (void) calculateValues
{
	if ( [values count] == 0 )
	{
		peakEntries		= 0;
		minValue		= 0;
		maxValue		= 100.0f;
		
		totalEntries	= 0;
		totalValue		= 0.0;
	}
	else 
	{
		peakEntries		= 0;
		minValue		= 1000000000.0f;
		maxValue		= -1000000000.0f;
		totalEntries	= 0;
		totalValue		= 0.0;
		
		for (Line * obj in values) 
		{
			// total the number of entries.
			totalEntries += obj.values.count;
			
			// work out the peak number of entries
			// this is used to work out the step
			// distance accross the graph.
			if ( obj.values.count > peakEntries )
			{
				peakEntries = obj.values.count;
			}

			// add all the values and work out the
			// min / max values contained
			for ( NSNumber * num in obj.values )
			{
				float value = [num floatValue];
				totalValue += value;
				if (value > maxValue) maxValue = value;
				if (value < minValue) minValue = value;
			}
		}
	}
	
	// ------------------------------------------
	// add ten percent, this makes sure the 
	// line does not touch the top of the graph
	// ------------------------------------------
	maxValue += maxValue * 0.1f;
	minValue += minValue * 0.1f;
	// ------------------------------------------
}

// -------------------------------------------------------------------
// Calculate the Line Height
// -------------------------------------------------------------------
- (float) calculateLineHeightFromRect:(CGRect)rect
{
	return rect.origin.y + ( ( minValue >= 0.0f ) ? rect.size.height - ((lineWidth > 1) ? lineWidth * 0.5f : 0) : rect.size.height * 0.5f ); // move the line to the center if we have negative values
}

// -------------------------------------------------------------------
// orientate the Context and fix the rect
// -------------------------------------------------------------------
- (CGRect) orientateContext:(CGContextRef)context AndRect:(CGRect)rect
{
	// ----------------------------------------
	// Deal with the device orientation
	// ----------------------------------------
	if (orientation == LineGraphOrientationLandscapeRight)
	{
		float width = rect.size.width;
		rect.size.width = rect.size.height;
		rect.size.height = width;
		
		// apply the a rotation to the context
		CGContextTranslateCTM( context, 0, rect.size.width );
		CGContextRotateCTM( context, -1.57079633 );
		
	}
	else if (orientation == LineGraphOrientationLandscapeLeft)
	{
		float width = rect.size.width;
		rect.size.width = rect.size.height;
		rect.size.height = width;
		
		// apply the a rotation to the context
		CGContextTranslateCTM( context, rect.size.height, 0 );
		CGContextRotateCTM( context, 1.57079633 );
	}
	else if (orientation == LineGraphOrientationPortraitUpsideDown)
	{
		// apply the a rotation to the context
		CGContextTranslateCTM( context, rect.size.width, rect.size.height );
		CGContextRotateCTM( context, 3.14159265 );
	}
	// ----------------------------------------
	
	return rect;
}

// -------------------------------------------------------------------
// Render the Graph Key in the context
// -------------------------------------------------------------------
- (CGRect) renderGraphKeyInContext:(CGContextRef)context AndRect:(CGRect)rect
{	
	// ---------------------------------------
	// work out the max text volume
	// ---------------------------------------
	CGSize maxSize = CGSizeMake(0, 0);
	NSInteger count = 0;
	for (Line * obj in values) 
	{
		if ( ( obj.name ) && 
			 ( [obj.name length] > 0 ) )
		{
			count++;
			
			CGSize size = [obj.name sizeWithFont:fontSmall];
			
			if ( size.width > maxSize.width )
			{
				maxSize.width = size.width;
			}
			if ( size.height > maxSize.height )
			{
				maxSize.height = size.height;
			}
		}		
	}
	// ---------------------------------------
	
	// Do we have anything to draw
	if ( count > 0 )
	{
		CGSize colorKeySize = CGSizeMake(5, 5); 
		float  edge			= 4;
		float  spacing      = 4;
		
		NSInteger keyPlacement = 0; // right center
		CGPoint step;
		
		// add in the size of the key object and the spacing
		maxSize.width += colorKeySize.width + ( spacing * 3 ); 
		
		CGRect area;
		switch(keyPlacement)
		{
			case 0: // center right
			{
				area.size.width = maxSize.width;
				area.size.height = ( maxSize.height * count ) + ( spacing * 2 );
				
				area.origin.x = ( rect.origin.x + rect.size.width ) - ( area.size.width );
				area.origin.y = ( rect.origin.y + ( rect.size.height * 0.5f ) ) - ( area.size.height * 0.5f );
				
				// work out the step
				step.x = 0;
				step.y = maxSize.height;
				
				// reduce the main draw area.
				rect.size.width -= ( area.size.width + edge );

				break;
			}
		}
		
		
		// ------------------------------------------------------------
		// Draw the bar rect
		// ------------------------------------------------------------
		CGContextSetFillColorWithColor(context,[fillColor CGColor]);
		CGContextFillRect(context, area);
		// ------------------------------------------------------------
		
		// ------------------------------------------------------------
		// Draw a line round the outside of the area
		// ------------------------------------------------------------
		// Work out the edge of the graph
		CGPoint min	 = area.origin;
		CGPoint max	 = CGPointMake( area.origin.x + area.size.width, area.origin.y + area.size.height );
		
		CGContextSetLineCap(context, kCGLineCapRound);
		CGContextSetLineJoin(context, kCGLineJoinRound);
		CGContextSetStrokeColorWithColor(context, [gridLineColor CGColor]);
		CGContextSetLineWidth(context, 1);	
		
		// start the drawing
		CGContextSetAllowsAntialiasing(context, NO);
		CGContextBeginPath(context);
		CGContextMoveToPoint(context, min.x, min.y );
		CGContextAddLineToPoint(context, min.x, max.y );
		CGContextAddLineToPoint(context, max.x, max.y );
		CGContextAddLineToPoint(context, max.x, min.y );
		CGContextAddLineToPoint(context, min.x, min.y );
		CGContextDrawPath(context, kCGPathStroke);	
		CGContextSetAllowsAntialiasing(context, YES);
		// ------------------------------------------------------------
		
		// ------------------------------------------------------------
		// Draw the key
		// ------------------------------------------------------------
		CGPoint start = area.origin;
		
		CGContextSelectFont(context, [fontSmall.fontName UTF8String], [fontSmall pointSize], kCGEncodingMacRoman);
		CGContextSetTextDrawingMode (context, kCGTextFill); 
		CGContextSetTextMatrix(context, lineTextTransform);
		
		CGRect box = CGRectMake(0, 0, colorKeySize.width, colorKeySize.height);
		for (Line * obj in values) 
		{
			if ( ( obj.name ) && 
				( [obj.name length] > 0 ) )
			{
				// render the box
				box.origin.x = start.x + spacing;
				box.origin.y = start.y + spacing + ( ( maxSize.height * 0.5f ) - ( box.size.height * 0.5f ) );
				CGContextSetFillColorWithColor(context,[obj.color CGColor]);
				CGContextFillRect(context, box);
								
				// draw the text.
				float x = start.x + (spacing * 2) + box.size.width;
				float y = start.y + maxSize.height;
				CGContextSetFillColorWithColor(context, [fontColor CGColor]);
				CGContextShowTextAtPoint(context, x, y, [obj.name UTF8String], obj.name.length);
				
				start.x += step.x;
				start.y += step.y;
			}
		}
		// ------------------------------------------------------------
	}	
	
	return rect;
}

// -------------------------------------------------------------------
// Render the Labels in the context
// -------------------------------------------------------------------
- (CGRect) renderLabelsInContext:(CGContextRef)context AndRect:(CGRect)rect
{	
	// Set the Context Font the custom one				
	CGContextSelectFont(context, [font.fontName UTF8String], [font pointSize], kCGEncodingMacRoman);
	
	//The following code is for Drawing Text
	CGContextSetTextDrawingMode (context, kCGTextFill); 
	
	// set the font color
	CGContextSetFillColorWithColor(context, [fontColor CGColor]);
	
	// if we have a title
	// render it accross the top
	if ( ( title ) && 
		 ( [title length] ) )
	{
		CGSize stringSize = [title sizeWithFont:font];
		
		int x   = rect.origin.x + ( rect.size.width * 0.5f ) - ( stringSize.width * 0.5f );
		int y   = rect.origin.y + ( stringSize.height * 0.5f );
		
		rect.origin.y    += stringSize.height;
		rect.size.height -= stringSize.height;
		
		// flip the context text matrix as fonts are rendered upsode down.
		CGContextSetTextMatrix(context, lineTextTransform);
		
		// draw the text.
		CGContextShowTextAtPoint(context, x, y, [title UTF8String], title.length);
	}
	
	// if we have a horizontal axis name
	// render it.
	if ( ( horizontalAxisName ) && 
		 ( [horizontalAxisName length] ) )
	{
		CGSize stringSize = [horizontalAxisName sizeWithFont:font];
		
		rect.size.height -= [font pointSize];
		
		int x   = rect.origin.x + ( rect.size.width * 0.5f ) - ( stringSize.width * 0.5f );
		int y   = rect.origin.y + rect.size.height + [font pointSize];
		
		// flip the context text matrix as fonts are rendered upsode down.
		CGContextSetTextMatrix(context, lineTextTransform);
		
		// draw the text.
		CGContextShowTextAtPoint(context, x, y, [horizontalAxisName UTF8String], horizontalAxisName.length);
	}
	
	// if we have a vertical axis name
	// render it
	if ( ( verticalAxisName ) &&
		 ( [verticalAxisName length] ) )
	{
		CGSize stringSize = [verticalAxisName sizeWithFont:font];
		int addition      = 4;
		
		rect.origin.x    += [font pointSize] + addition;
		rect.size.width  -= [font pointSize] + addition;
		
		int x   = rect.origin.x - addition;
		int y   = rect.origin.y + ( rect.size.height* 0.5f ) + ( stringSize.width * 0.5f );
		
		// flip the context text matrix as fonts are rendered upsode down.
		// rotate round 90 dgrees
		CGContextSetTextMatrix(context, lineTextRotTransform);
		
		// draw the text.
		CGContextShowTextAtPoint(context, x, y, [verticalAxisName UTF8String], verticalAxisName.length);
	}
	
	return rect;
}

// -------------------------------------------------------------------
// render an array of labels in the context
// -------------------------------------------------------------------
- (CGRect) renderArrayLabelsInContext:(CGContextRef)context AndRect:(CGRect)rect
{
	if ( labels && [labels count] )
	{
		rect.size.height -= [font pointSize] + 5;
		
		float areawidth		= rect.size.width;
		float maxLineWidth	= 0;
		for (Line * obj in values) 
		{
			if ( obj.width > maxLineWidth )
			{
				maxLineWidth = obj.width;
			}
		}
		areawidth -= maxLineWidth;
		
		// limit the spread of the points
		NSInteger entries = peakEntries;
		if ( minNumEntries > peakEntries )
		{
			entries = minNumEntries;
		}
		else if ( peakEntries > maxNumEntries )
		{
			entries = maxNumEntries;
		}
		
		float stepDistance = areawidth / ( entries - 1 );
		
		float startX = rect.origin.x;
		if ( drawScale )
		{
			NSString * zero = @"0";
			CGSize zeroSize = [zero sizeWithFont:font];
			
			int addition      = 2;
			if ( zeroSize.width > [font pointSize] )
			{
				startX    += zeroSize.width + addition;
			}
			else
			{
				startX    += [font pointSize] + addition;
			}
		}
		
		for ( NSString * txt in labels )
		{
			CGSize stringSize = [txt sizeWithFont:font];
			
			int x   = startX - ( stringSize.width * 0.5f );
			int y   = rect.origin.y + rect.size.height + [font pointSize];
			
			startX += stepDistance;
			
			// flip the context text matrix as fonts are rendered upsode down.
			CGContextSetTextMatrix(context, lineTextTransform);
			
			// draw the text.
			CGContextShowTextAtPoint(context, x, y, [txt UTF8String], txt.length);
		}
		
		rect.size.height -= 5;
	}
	return rect;
}

// -------------------------------------------------------------------
// Render the scale values in the context
// -------------------------------------------------------------------
- (CGRect) renderScaleValuesInContext:(CGContextRef)context AndRect:(CGRect)rect
{
	NSString * zero = @"0";
	CGSize zeroSize = [zero sizeWithFont:font];
	
	int addition      = 2;
	if ( zeroSize.width > [font pointSize] )
	{
		rect.origin.x    += zeroSize.width + addition;
		rect.size.width  -= zeroSize.width + addition;
	}
	else
	{
		rect.origin.x    += [font pointSize] + addition;
		rect.size.width  -= [font pointSize] + addition;
	}
	
	// Set the Context Font the custom one				
	CGContextSelectFont(context, [font.fontName UTF8String], [font pointSize] - 2, kCGEncodingMacRoman);
	
	//The following code is for Drawing Text
	CGContextSetTextDrawingMode (context, kCGTextFill); 
	
	// set the font color
	CGContextSetFillColorWithColor(context, [fontColor CGColor]);
		
	// --------------------------------------------
	// render the max value
	// --------------------------------------------
	{
		NSString * strMax = (maxValue > 1) ?  [NSString stringWithFormat:@"%d", (int)maxValue] : [NSString stringWithFormat:@"%f", maxValue];
		CGSize size = [strMax sizeWithFont:font];
		
		int x   = rect.origin.x - 2;
		int y   = rect.origin.y + size.width;
		
		// flip the context text matrix as fonts are rendered upsode down.
		CGContextSetTextMatrix(context, lineTextRotTransform);
		
		// draw the text.
		CGContextShowTextAtPoint(context, x, y, [strMax UTF8String], strMax.length);
	}
	// --------------------------------------------
	
	// --------------------------------------------
	// render the min value if less that zero
	// --------------------------------------------
	if ( minValue < 0 )
	{
		NSString * strMax = (fabs(minValue) > 1) ?  [NSString stringWithFormat:@"%d", (int)minValue] : [NSString stringWithFormat:@"%f", minValue];
		
		int x   = rect.origin.x - 2;
		int y   = rect.origin.y + rect.size.height;
		
		// flip the context text matrix as fonts are rendered upsode down.
		CGContextSetTextMatrix(context, lineTextRotTransform);
		
		// draw the text.
		CGContextShowTextAtPoint(context, x, y, [strMax UTF8String], strMax.length);
	}
	// --------------------------------------------
	
	// --------------------------------------------
	// render the zero mark
	// --------------------------------------------
	{	
		float lineHeight = [self calculateLineHeightFromRect:rect];
		
		int x   = rect.origin.x - ( zeroSize.width + 2 );
		int y   = lineHeight + ( [font pointSize] * 0.5f );
		
		// flip the context text matrix as fonts are rendered upsode down.
		CGContextSetTextMatrix(context, lineTextTransform);
		
		// draw the text.
		CGContextShowTextAtPoint(context, x, y, [zero UTF8String], zero.length);
	}
	// --------------------------------------------
	
	
	return rect;
}

// -------------------------------------------------------------------
// Render the graph background in the context
// -------------------------------------------------------------------
- (void)   renderGraphBackGroundInContext:(CGContextRef)context AndRect:(CGRect)rect
{
	// ----------------------------------------
	// draw the fill area this is the bit 
	// directly behind the graph
	// ----------------------------------------
	if ( drawFill )
	{		
		// ------------------------------------------------------------
		// Draw the bar rect
		// ------------------------------------------------------------
		CGContextSetFillColorWithColor(context,[fillColor CGColor]);
		CGContextFillRect(context, rect);
		// ------------------------------------------------------------
	}
	// ----------------------------------------
	
	// ----------------------------------------
	// draw a grid
	// ----------------------------------------
	if ( drawGrid )
	{
		// Work out the edge of the graph
		CGPoint min	 = rect.origin;
		CGPoint max	 = CGPointMake( rect.origin.x + rect.size.width, rect.origin.y + rect.size.height );
		
		CGContextSetAllowsAntialiasing(context, NO);
		
		// start the drawing
		CGContextBeginPath(context);
		
		CGContextSetStrokeColorWithColor(context, [gridLineColor CGColor]);
		
		// set the line width
		CGContextSetLineWidth(context, gridLineWidth);
		
		CGContextMoveToPoint(context, min.x, min.y );
		CGContextAddLineToPoint(context, min.x, max.y );
		
		int lines = 10;
		float height = min.y;
		float stepHeight = rect.size.height / lines;
		for ( int i=0; i<lines-1; i++ )
		{
			height += stepHeight;
			CGContextMoveToPoint(context, min.x, height );
			CGContextAddLineToPoint(context, max.x, height );
		}	
		
		CGContextDrawPath(context, kCGPathStroke);
		
		CGContextSetAllowsAntialiasing(context, YES);
	}
	
	if ( drawEdge )
	{
		// Work out the edge of the graph
		CGPoint min	 = rect.origin;
		CGPoint max	 = CGPointMake( rect.origin.x + rect.size.width, rect.origin.y + rect.size.height );
		
		CGContextSetAllowsAntialiasing(context, NO);
		
		// start the drawing
		CGContextBeginPath(context);
		
		// set the color
		CGContextSetStrokeColorWithColor(context, [gridLineColor CGColor]);
		
		// set the line width
		CGContextSetLineWidth(context, gridLineWidth);
		
		CGContextMoveToPoint(context, min.x, min.y );
		CGContextAddLineToPoint(context, min.x, max.y );
		CGContextAddLineToPoint(context, max.x, max.y );
		CGContextAddLineToPoint(context, max.x, min.y );
		CGContextAddLineToPoint(context, min.x, min.y );
		
		CGContextDrawPath(context, kCGPathStroke);
		
		CGContextSetAllowsAntialiasing(context, YES);
	}
	// ----------------------------------------
	
}

// -------------------------------------------------------------------
// Render the graph Marker Line in the context
// -------------------------------------------------------------------
- (void) renderGraphMarkerLineInContext:(CGContextRef)context AndRect:(CGRect)rect
{
	// ----------------------------------------
	// draw the background area
	// ----------------------------------------
	if ( drawLine )
	{
		// ----------------------------------------
		// Work out were the horazontal line should
		// sit.
		// ----------------------------------------
		float lineHeight = [self calculateLineHeightFromRect:rect];
		// ----------------------------------------
		
		// Work out the edge of the graph
		CGPoint verticalStart	 = rect.origin;
		CGPoint verticalEnd		 = CGPointMake( verticalStart.x, verticalStart.y + rect.size.height );
		
		// Work out the line of the
		CGPoint horazontalStart  = CGPointMake( rect.origin.x, lineHeight );
		CGPoint horazontalEnd    = CGPointMake( rect.origin.x + rect.size.width, lineHeight );
		
		// start the drawing
		CGContextBeginPath(context);
		
		// set the color of the stroke
		CGContextSetStrokeColorWithColor(context, [lineColor CGColor]);
		
		// set the line width
		CGContextSetLineWidth(context, lineWidth);
		// turn off antialasing
		CGContextSetAllowsAntialiasing(context, NO);
		
		// draw the vertical line
		CGContextMoveToPoint(context, verticalStart.x, verticalStart.y-1);
		CGContextAddLineToPoint(context, verticalEnd.x, verticalEnd.y );
		
		// draw the horazontal line
		CGContextMoveToPoint(context, horazontalStart.x, horazontalStart.y);
		CGContextAddLineToPoint(context, horazontalEnd.x, horazontalEnd.y );
		
		CGContextDrawPath(context, kCGPathStroke);
		
		CGContextSetAllowsAntialiasing(context, YES);
	}
	// ----------------------------------------
}

#pragma mark ---------------------------------------------------------
#pragma mark === End Private Functions  ===
#pragma mark ---------------------------------------------------------

@end
