//
//  WebtrendsClientLibrary.h
//  
//
//  Copyright 2009-2010 Webtrends. All rights reserved.
//

#import <UIKit/UIKit.h>
#import <Foundation/Foundation.h>


#pragma mark -
#pragma mark WTAppDelegate

/*
 
 If you would like automatic event generation for application start, stop and notification methods, 
 change the base class of your app delegate from NSObject to WTAppDelegate. If you override any of 
 these methods and still want automatic behavior, be sure to call super. These methods will automatically
 send events to Webtrends for data collection. 
 
*/




@interface WTAppDelegate: NSObject  {
	
}

/*
 
 Sends an application/start event to Webtrends automatically. The parameters
 will indicate that the app was launched from the device's home screen.
 
*/
- (void)applicationDidFinishLaunching:(UIApplication *)application;

/*
 
 Sends an application/start event to Webtrends automatically. The parameters
 will indicate that the app was launched either from a notification or from
 a registered URL  (whichever applies).
*/

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions;

/*

 Sends an application/notification event to Webtrends automatically.
 
*/
- (void)application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo;

/*

 Sends an application/exit event to Webtrends automatically. In most cases the event will
 be queued and sent to Webtrends the next time the application starts, as there
 may not be time to send it before the app terminates.
 
*/
- (void)applicationWillTerminate:(UIApplication *)application;


@end

#pragma mark -
#pragma mark WTDC

/*

 The WTDC category is added to NSObject. You can call these methods from any subclass 
 of NSObject to send data collection events to Webtrends. Typically this would be done
 from your view controllers. You can create the WTEvent object using the convenience methods
 (see class methods for WTEvent), set your own parameter values or implement the 
 WTDCEventSource protocol and populate the event parameters in that method.
 
*/

@class WTEvent;

@interface NSObject (WTDC) 

-(void) trackEvent:(WTEvent*) event;
-(void) disableEventTracking;
-(void) enableEventTracking;
-(void) shutdownEventTracking;
-(void) clearEventQueue;
-(void) suspendWebtrendsNotifications;
-(void) resumeWebtrendsNotifications;

@property BOOL DCSVerbose;
@property BOOL DCSDebug;
@property (readonly) NSNumber *queuedEventCount;


@end // WTEvent

#pragma mark -
#pragma mark WTDCEventSource

/*

 trackEvent will invoke this callback if it's present in the sending class. 
 This method may return additional parameters that should be sent along with the 
 event to Webtrends by returning a dictionary with those values. 
 This is a convenient way to manage (most) of your tracking logic from one place.
 
*/

@protocol WTDCEventSource

- (NSDictionary*) parametersForEvent:(WTEvent*) event;

@end // WTEvent


#pragma mark -
#pragma mark WTParameter

/*

 The WTParameter enumeration contains all supported
 parameter types (typically used in calls to WTEvent's setValue:forParameter: method). 
 These will automatically map to the underlying Webtrends parameter. If the
 parameter is not available here you may use a custom parameter using
 WTEvent's setValue:forCustomParameter: method.
 
*/
typedef enum {
	WTParameterApplicationVersion = 0,
	WTParameterContentGroup,
	WTParameterMediaEvent,
	WTParameterMediaName,
	WTParameterMediaType,
	WTParameterConversionName,
	WTParameterConversionStep,
	WTParameterEventTimestamp,
	WTParameterCustomEvent,
	WTParameterGeographicLocation,
	WTParameterTitle,
	WTParameterPageOfInterest,
	WTParameterProductId,
	WTParameterProductSku,
	WTParameterSpecialEvent,
	WTParameterScreenResolution,
	WTParameterConnectionType,
	WTParameterErrorMessage,
	WTParameterUnknown
} WTParameter;

/*
 
 WTLaunchStyle may be used when calling WTEvent's eventForAppLaunch
 method. 
 
*/
typedef enum {
	WTLaunchStyleFromURL,
	WTLaunchStyleFromNotification,
	WTLaunchStyleFromHomeScreen
} WTLaunchStyle;


typedef enum {
	WTUseCustomId,
	WTUseWebtrendsId,
	WTUseSystemId
} WTVisitorType;


extern NSString *WTDCEventSentNotification; 

@interface WTEvent : NSObject {
	
	NSString *eventPath; 
	NSString *eventDescription; 
	NSMutableDictionary *_p;
	NSMutableString *_descriptionStorage;
}

#pragma mark -
#pragma mark WTEvent - Initialization Methods

/*

 Initializes a WTEvent with a path and description. Both are required or a
 NSInternalInconsistencyException will be thrown. The event type is optional.
 
*/
- (id) initWithEventPath:(NSString*) eventPath eventDescr:(NSString*) eventDescr;
- (id) initWithEventPath:(NSString*) eventPath eventDescr:(NSString*) eventDescr eventType: (NSString*) eventType;

#pragma mark -
#pragma mark WTEvent - Parameter Methods

/*
 
 Sets the value for a built-in Webtrends parameter. 
 If the parameter is invalid the value will not be set.
 
*/
- (void) setValue:(NSString*) value forParameter:(WTParameter) parameter;

/*
 
 Sets the value for a custom parameter. 
 If parameter is nil the value will not be set.

*/
- (void) setValue:(NSString*) value forCustomParameter:(NSString*) parameter;

#pragma mark -
#pragma mark WTEvent - Convenience Methods


/*
 
 Creates a new WTEvent initialized with the event path, description and type. The former two parameters
 are requried or an NSInternalInconsistencyException will be thrown.

*/
+ (WTEvent*) eventWithPath:(NSString*) eventPath eventDescr:(NSString*) eventDescr eventType: (NSString*) eventType;

/*
 
 Returns a new WTEvent that indicates an error has occured in the application. An optional message may
 be specified that will display in reports. 
 
*/
+ (WTEvent*) eventForError:(NSString*) message;

/*

 Returns a new WTEvent that indicates the application was launched. The WTLaunchStyle indicates how
 the application was started--either from the home screen, from a push notification or a registered URL.
 
*/
+ (WTEvent*) eventForAppLaunch:(WTLaunchStyle) launchStyle; 

/*
 
 Returns a new WTEvent that indicates the application has exited (normally). 
 
*/
+ (WTEvent*) eventForAppExit;

/*
 
 Returns a new WTEvent that indicates a notification was recieved.
 
*/
+ (WTEvent*) eventForNotification;

/*
 
 Returns a new WTEvent that indicates a product was viewed. The path,
 description and product sku are requried or an NSInternalInconsistencyException will be thrown.

*/
+ (WTEvent*) eventForProductView:(NSString*) eventPath eventDescr:(NSString*) eventDescr eventType: (NSString*) eventType contentGroup:(NSString*) contentGroup productId:(NSString*) productId productSku:(NSString*) productSku;

/*

 Returns a new WTEvent that indicates a product was viewed. The path and
 description are requried or an NSInternalInconsistencyException will be thrown.
 
*/
+ (WTEvent*) eventForMediaView:(NSString*) eventPath eventDescr:(NSString*) eventDescr eventType: (NSString*) eventType contentGroup:(NSString*) contentGroup mediaName:(NSString*) mediaName mediaType:(NSString*) mediaType mediaEvent:(NSString*) mediaEvent;

/*
 
 Returns a new WTEvent that indicates a conversion event occured. The path,
 description and conversion name are requried or an NSInternalInconsistencyException will be thrown.

*/
+ (WTEvent*) eventForConversion:(NSString*) eventPath eventDescr:(NSString*) eventDescr eventType: (NSString*) eventType contentGroup:(NSString*) contentGroup conversionName:(NSString*) conversionName;


/*
 
 Returns a new WTEvent that indicates a content view event occured. The path and
 description are requried or an NSInternalInconsistencyException will be thrown.
 
*/
+ (WTEvent*) eventForContentView:(NSString*) eventPath eventDescr:(NSString*) eventDescr eventType: (NSString*) eventType contentGroup:(NSString*) contentGroup;


/*
 
 Returns a new WTEvent that indicates an "action" event occured. These events are not counted
 as page views and typically indicate than some user activity occured (as opposed to a "view").
 The path and description are requried or an NSInternalInconsistencyException will be thrown.
 
*/
+ (WTEvent*) eventForAction:(NSString*) eventPath eventDescr:(NSString*) eventDescr eventType: (NSString*) eventType;


+ (int) descriptionLength;


#pragma mark -
#pragma mark WTEvent - Properties

@property(nonatomic, retain) NSString *eventPath;
@property(nonatomic, retain) NSString *eventDescription;

#pragma mark -
@end // WTEvent
